/**
 * @file
 * VuoList_VuoFont C type definition.
 *
 * @copyright Copyright © 2012–2016 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VuoList_VuoFont_H
#define VuoList_VuoFont_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoList_VuoFont VuoList_VuoFont
 * A list of @ref VuoFont elements.
 *
 * @{
 */

// This header is generated by vuo/type/list/generateVariants.sh.

#if defined(DOXYGEN) || !defined(VuoList_VuoFont_TYPE_DEFINED)
/// @{
/**
 * A list of @ref VuoFont elements.
 */
typedef const struct VuoList_VuoFont_struct { void *l; } * VuoList_VuoFont;
/// @}
#endif

/**
 * Creates a new list of @ref VuoFont elements.
 */
VuoList_VuoFont VuoListCreate_VuoFont(void);

/**
 * Creates a new list of `count` instances of `value`.
 *
 * Use this in conjunction with @ref VuoListGetData_VuoFont to quickly initialize a large list.
 */
VuoList_VuoFont VuoListCreateWithCount_VuoFont(const unsigned long count, const VuoFont value);

/**
 * Makes a shallow copy of `list` — its items are retained (not copied) by the new list.
 */
VuoList_VuoFont VuoListCopy_VuoFont(const VuoList_VuoFont list);

/**
 * Returns the @ref VuoFont at @c index.
 * Index values start at 1.
 * If the list has no items, returns a default value.
 * Attempting to access an out-of-bounds index returns the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
VuoFont VuoListGetValue_VuoFont(const VuoList_VuoFont list, const unsigned long index);

/**
 * Returns a pointer to a C array containing the list items.
 *
 * Use this if you need a fast way to get or change multiple list items.
 *
 * You can modify values in the list by changing them in this array.
 * Just don't attempt to access beyond the list size.
 *
 * The pointer becomes invalid if you modify the list size using other functions
 * (e.g., Insert, Prepend, Append, Cut, Remove);
 * if you use those functions, just get a new pointer by calling this function again.
 *
 * If the list has no items, returns NULL.
 *
 * The pointer remains owned by the list; don't free it.
 */
VuoFont *VuoListGetData_VuoFont(const VuoList_VuoFont list);

/**
 * Changes the @ref VuoFont at @c index.
 * Index values start at 1.
 * If the list has no items, nothing is changed.
 * Attempting to change an out-of-bounds index changes the first item in the list (if the index is 0), or last item in the list (if the index is greater than the list size).
 */
void VuoListSetValue_VuoFont(const VuoList_VuoFont list, const VuoFont value, const unsigned long index);

/**
 * Inserts the `VuoFont` immediately before `index`.
 * Index values start at 1.
 * Inserting at index 0 prepends the value to the list.
 * Inserting at an index beyond the last value in the list appends the value to the list.
 */
void VuoListInsertValue_VuoFont(const VuoList_VuoFont list, const VuoFont value, const unsigned long index);

/**
 * Prepends @c value to @c list.
 */
void VuoListPrependValue_VuoFont(VuoList_VuoFont list, const VuoFont value);

/**
 * Appends @c value to @c list.
 */
void VuoListAppendValue_VuoFont(VuoList_VuoFont list, const VuoFont value);

/**
 * Swaps the value at `indexA` with the value at `indexB`.
 */
void VuoListExchangeValues_VuoFont(VuoList_VuoFont list, const unsigned long indexA, const unsigned long indexB);

#ifdef VuoFont_SUPPORTS_COMPARISON
/**
 * Sorts `list`.
 */
void VuoListSort_VuoFont(VuoList_VuoFont list);

/**
 * Returns true if the two lists are equivalent.
 *
 * NULL lists are never equal to non-NULL lists (even empty lists).
 */
bool VuoList_VuoFont_areEqual(const VuoList_VuoFont a, const VuoList_VuoFont b);

/**
 * Returns true if `a` < `b`.
 */
bool VuoList_VuoFont_isLessThan(const VuoList_VuoFont a, const VuoList_VuoFont b);
#endif

/**
 * Generates a random permutation of `list`.
 *
 * `chaos` ranges from 0 to 1.  When `chaos` is 1, a full Fisher–Yates shuffle is performed.  When less than 1, fewer iterations are performed.
 */
void VuoListShuffle_VuoFont(VuoList_VuoFont list, const double chaos);

/**
 * Reverses the order of the items in `list`.
 */
void VuoListReverse_VuoFont(VuoList_VuoFont list);

/**
 * Removes items from the list except those in the range specified by `startIndex` and `itemCount`.
 */
void VuoListCut_VuoFont(VuoList_VuoFont list, const signed long startIndex, const unsigned long itemCount);

/**
 * Removes the first value from @c list.
 */
void VuoListRemoveFirstValue_VuoFont(VuoList_VuoFont list);

/**
 * Removes the last value from @c list.
 */
void VuoListRemoveLastValue_VuoFont(VuoList_VuoFont list);

/**
 * Removes all values from @c list.
 */
void VuoListRemoveAll_VuoFont(VuoList_VuoFont list);

/**
 * Removes the `VuoFont` at `index`.
 * Index values start at 1.
 * Attempting to remove index 0 or an index beyond the last value in the list has no effect.
 */
void VuoListRemoveValue_VuoFont(VuoList_VuoFont list, const unsigned long index);

/**
 * Returns the number of elements in @c list.
 */
unsigned long VuoListGetCount_VuoFont(const VuoList_VuoFont list);

/**
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *	["uno", "dos", "tres", "catorce"]
 * }
 */
VuoList_VuoFont VuoList_VuoFont_makeFromJson(struct json_object * js);

/**
 * Encodes @c value as a JSON object.
 */
struct json_object * VuoList_VuoFont_getJson(const VuoList_VuoFont value);

/**
 * Produces a brief human-readable summary of @c value.
 */
char * VuoList_VuoFont_getSummary(const VuoList_VuoFont value);

/// @{
/**
 * Automatically generated function.
 */
VuoList_VuoFont VuoList_VuoFont_makeFromString(const char *str);
char * VuoList_VuoFont_getString(const VuoList_VuoFont value);
/// @}

/**
 * @}
 */

#endif
